"""An input field that must be unique in the database."""

from typing import Annotated, TypeVar

from pydantic import AfterValidator
from pydantic_core.core_schema import ValidationInfo

from gso.services import subscriptions

T = TypeVar("T")


def validate_field_is_unique(value: T, info: ValidationInfo) -> T:
    """Validate that a field is unique."""
    if len(subscriptions.get_active_subscriptions_by_field_and_value(str(info.field_name), str(value))) > 0:
        msg = f"{info.field_name} must be unique"
        raise ValueError(msg)
    return value


UniqueField = Annotated[T, AfterValidator(validate_field_is_unique)]
